@@ -2,7 +2,7 @@ import inspect
 import pathlib
 from functools import lru_cache
 from io import TextIOBase
-from typing import Sequence, Union, TypeVar, Optional, Literal
+from typing import Sequence, Union, TypeVar, Optional, Literal, Iterable
 
 import polars as pl
 from black import Mode, format_str
@@ -193,11 +193,12 @@ class ExprTool:
 
         return self.exprs_list
 
-    def dag(self, merge: bool, date, asset):
+    def dag(self, merge: bool, skip_columns, date, asset):
         """生成DAG"""
         G = dag_start(self.exprs_list, self.get_current_func, self.get_current_func_kwargs, date, asset)
         if merge:
-            G = dag_middle(G, self.exprs_names, self.get_current_func, self.get_current_func_kwargs, date, asset)
+            G = dag_middle(G, self.exprs_names, skip_columns, self.get_current_func, self.get_current_func_kwargs, date,
+                           asset)
         return dag_end(G)
 
     def all(self, exprs_src, style: Literal['pandas', 'polars', 'sql'] = 'polars',
@@ -207,8 +208,9 @@ class ExprTool:
             extra_codes: Sequence[object] = (),
             over_null: Literal['order_by', 'partition_by', None] = 'partition_by',
             table_name: str = 'self',
-            filter_last: bool = False,
+            ge_date_idx: int = 0,
             skip_simplify: bool = False,
+            skip_columns: Iterable[str] = (),
             **kwargs):
         """功能集成版，将几个功能写到一起方便使用
 
@@ -233,8 +235,10 @@ class ExprTool:
         extra_codes: Sequence[object]
             需要复制到模板中的额外代码
         table_name
-        filter_last
+        ge_date_idx
         skip_simplify
+        skip_columns
+            数据中已经存在的列不再参与计算，直接使用历史值
 
         Returns
         -------
@@ -253,7 +257,7 @@ class ExprTool:
         # 提取公共表达式
         self.cse(exprs_dst, symbols_repl=numbered_symbols('_x_'), exprs_src=exprs_src)
         # 有向无环图流转
-        exprs_ldl, G = self.dag(True, date, asset)
+        exprs_ldl, G = self.dag(True, skip_columns, date, asset)
 
         if regroup:
             exprs_ldl.optimize(merge=style != 'sql')
@@ -275,7 +279,7 @@ class ExprTool:
                         extra_codes=extra_codes,
                         over_null=over_null,
                         table_name=table_name,
-                        filter_last=filter_last,
+                        ge_date_idx=ge_date_idx,
                         **kwargs)
 
         logger.info(f'{style} code is generated')
@@ -297,8 +301,9 @@ class ExprTool:
                   date: str = 'date', asset: str = 'asset',
                   over_null: Literal['order_by', 'partition_by', None] = 'partition_by',
                   table_name: str = 'self',
-                  filter_last: bool = False,
+                  ge_date_idx: int = 0,
                   skip_simplify: bool = False,
+                  skip_columns: Iterable[str] = (),
                   **kwargs) -> str:
         """通过字符串生成代码， 加了缓存，多次调用不重复生成"""
         raw, exprs_list = sources_to_exprs(self.globals_, source, *more_sources, convert_xor=convert_xor)
@@ -314,8 +319,9 @@ class ExprTool:
                                           ),
                              over_null=over_null,
                              table_name=table_name,
-                             filter_last=filter_last,
+                             ge_date_idx=ge_date_idx,
                              skip_simplify=skip_simplify,
+                             skip_columns=skip_columns,
                              **kwargs)
 
         # 移回到cache，防止多次调用多次保存
@@ -378,8 +384,9 @@ def codegen_exec(df: Union[DataFrame, None],
                  template_file: Optional[str] = None,
                  date: str = 'date', asset: str = 'asset',
                  table_name: str = 'self',
-                 filter_last: bool = False,
+                 ge_date_idx: int = 0,
                  skip_simplify: bool = False,
+                 skip_columns: Iterable[str] = (),
                  **kwargs) -> Union[DataFrame, str]:
     """快速转换源代码并执行
 
@@ -419,11 +426,17 @@ def codegen_exec(df: Union[DataFrame, None],
         - None: 不做处理
     table_name:str
         表名。只在style参数为sql时有效
-    filter_last:bool
-        在实盘时，只需要最后一天日期的数据，可以在最后一个`ts`之后过滤数据。目前只在style参数为'polars', 'pandas'时有效
+    ge_date_idx:int
+        在实盘时，只需要最后一两天日期的数据，可以在最后一个`ts`之后过滤数据。目前只在style参数为'polars', 'pandas'时有效
+        0 表示不过滤 >=date[0]
+        -1 表示最新一天 >=date[-1]
+        -2 表示最近两天 >=date[-2]
     skip_simplify:bool
         遗传算法时很有可能出现OPEN/OPEN，可以跳过化简步骤
-
+    skip_columns:
+        已经存在的列不参与计算。可用于加快计算速度。只在计算耗时久时再用，否则没有必要
+        例如：在研发阶段，第一次计算100个因子，第二次，只改动了其中的5个，所以只要将这5个从df.columns中排除即可。
+        注意：生成的源代码有差异。
 
     Returns
     -------
@@ -440,6 +453,8 @@ def codegen_exec(df: Union[DataFrame, None],
     2. 通过代码得到了函数 `get func from code`
 
     """
+    skip_columns = tuple(skip_columns)
+
     if df is not None:
         input_file = None
         # 以下代码都带缓存功能
@@ -451,12 +466,12 @@ def codegen_exec(df: Union[DataFrame, None],
 
         if input_file is not None:
             if input_file.endswith('.py'):
-                return _get_func_from_file_py(input_file)(df, filter_last)
+                return _get_func_from_file_py(input_file)(df, ge_date_idx)
             elif input_file.endswith('.sql'):
                 with pl.SQLContext(frames={table_name: df}) as ctx:
                     return ctx.execute(_get_code_from_file(input_file), eager=isinstance(df, _pl_DataFrame))
             else:
-                return _get_func_from_module(input_file)(df, filter_last)  # 可断点调试
+                return _get_func_from_module(input_file)(df, ge_date_idx)  # 可断点调试
     else:
         pass
 
@@ -476,8 +491,9 @@ def codegen_exec(df: Union[DataFrame, None],
         date=date, asset=asset,
         over_null=over_null,
         table_name=table_name,
-        filter_last=filter_last,
+        ge_date_idx=ge_date_idx,
         skip_simplify=skip_simplify,
+        skip_columns=skip_columns,
         **kwargs
     )
 
@@ -489,4 +505,4 @@ def codegen_exec(df: Union[DataFrame, None],
             return ctx.execute(code, eager=isinstance(df, _pl_DataFrame))
     else:
         # 代码一样时就从缓存中取出函数
-        return _get_func_from_code_py(code)(df, filter_last)
+        return _get_func_from_code_py(code)(df, ge_date_idx)
